print("SymmetricFunctionsinSuperSpace v 2.3.2 - Feb 16, 2018")
# v1.0 17/09/04 - initial implementation
# v1.1 17/09/05 - added coproduct (by Power basis) 
# v1.2 17/09/05 - added antipode (by Power basis)
# v1.3 17/10/14 - added Schur basis (still needs work) 
# v1.3.1 - fixed a bug with to_complete_on_basis returning 1 instead of h.one()
# v1.3.2 - added some documentation on the Schur basis
# v1.3.3 - fixed the sign and the order in which the antisymmetric part is multiplied (is it right now?)
# v1.4 - recognized that I need to name Schur basis as ss rather than s because there are 4 flavors
# v1.4.1 - corrected the .Schur_s class
# v1.5 - added the Schur_b basis (Schur bar) it is (roughly) omega applied to Schur_s
# v1.5.1 - documentation corrections
# v1.5.2 17/10/22 - started using the add_horizontal_border_strip method from within SuperPartitions
# v2.0 17/10/28 - generators are indexed by positive and negative integers (as the QSFSS), corrections to documentation
# v2.0.1 17/10/28 - expanded SP( to SuperPartition(, removed the load functions
# v2.0.2 17/10/28 - documentation on the antipode and coproduct
# v2.0.3 17/10/28 - fixed documentation on the ss and sb bases
# v2.0.4 17/10/28 - fixed documentation on algebra_morphism
# v2.1 17/11/05 - corrected Schur-bar basis to be omega applied the Schur-star
# v2.1.1 17/11/7 - corrections to the Schur-bar basis
# v2.2 17/11/8 - added the one true Schur basis by omega of Schur-*-bar basis
# v2.2.1 18/1/13: fixed input of elements like p["123"] (no longer possible)
# v2.2.2 18/1/18: fixed bug in basis construction f[a] where f is a basis and a is an integer
# v2.2.3 18/1/24: fixed bug where Schur bases had wrong display name
# v2.2.4 18/1/24: fixed bug with antipode (on power) sign was wrong
# v2.3 18/2/15 - added all_bi_degrees
# v2.3.1 18/2/15 - added skew_by (by conversion to the power sum basis), and homogeneous_bi_degree_component
# v2.3.2 18/2/16 - added doc for skew_by, fixed doctest for algebra_generators
# also changed bi_degree so that it computes it in SuperPartitions
# TODO: documentation for sign_matching and m_m_mult_fixed_a and better
# documentation for other basis
# TODO: put in examples of the use of SuperPartitions.options.display = "list" and "pair"
r"""
Symmetric functions in super space

Symmetric functions in super space is an algebra whose bases are indexed by 
``SuperPartitions``.

Let `P_{\Theta_n, X_n}` be the polynomial ring in two sets of variables
`Theta_n = \theta_1, theta_2, \ldots, \theta_n` and
`X_n = x_1, x_2, \ldots, x_n`
where the first set of variables anti-commute and the second set
of variables commute and commutes with the first set.  The symmetric group 
acts diagonally on this polynomial ring and the symmetric functions in 
superspace are isomorphic to the invariants in this polynomial ring.

``SummetricFunctionsinSuperSpace`` is isomorphic to the graded ring with
generators `p_{(0;)}, p_{(;1)}, p_{(1;)}, p_{(;2)}, p_{(2;)}, \ldots`
satisfying the relations

.. MATH::

    p_{(;i)} p_{(;j)} = p_{(;j)} p_{(;i)} \quad
    p_{(i;)} p_{(;j)} = p_{(;j)} p_{(i;)} \quad
    p_{(i;)} p_{(j;)} = - p_{(j;)} p_{(i;)}

There are three natural sets of generators that are analogues of the power,
complete and elementary bases of the symmetric functions.  They are indexed
by the super partition `(; n)` and `(n; )`.  Define the generators

.. MATH::

    p_{(; n)} = p_n \quad e_{(; n)} = e_n \quad h_{(; n)} = h_n

    p_{(; n)} = \sum_{i\geq1} x_i^n

    e_{(; n)} = \sum_{1 \leq i_1<i_2<\cdots<i_n} x_{i_1} x_{i_2} \cdots x_{i_n}

    h_{(; n)} = \sum_{1 \leq i_1\leq i_2\leq\cdots\leq i_n} x_{i_1} x_{i_2} \cdots x_{i_n}

where `p_n, e_n` and `h_n` are the symmetric function elements in the `X_n`
variables.  In addition there are fermonionic generators which include variables
from `Theta_n`.  These are given

.. MATH::

    p_{(n; )} = \sum_{i\geq1} \theta_i x_i^n

    e_{(n; )} = \sum_{i\geq1} \sum_{J:|J|=n,i \notin J} \theta_i x_{j_1} x_{j_2} \cdots x_{j_n}

The complete fermionic generator is most clearly expressed in terms of the monomial
basis (discussed below). 

The space of symmetric functions in super space is bigraded by the bosonic and
fermionic degree.

::

    sage: SFSS = SymmetricFunctionsinSuperSpace(QQ)
    sage: (m,p,e,h,ss,sb) = (SFSS.m(),SFSS.p(),SFSS.e(),SFSS.h(),SFSS.ss(),SFSS.sb())
    sage: all(b1(b2(b1([[3,2],[1,1]]))) == b1([[3,2],[1,1]])
    ....:     for b1 in [m,p,e,h,ss,sb] for b2 in [m,p,e,h,ss,sb] if b1!=b2)

REFERENCES:

    .. [DLM06] P. Desrosiers, L. Lapointe, P. Mathieu, Classical
        symmetric functions in superspace, J. Algebr Comb. (2006) 24:209--238,
        :arXiv:`0509408`

    .. [JL16] M. Jones, L. Lapointe, Pieri rules for Schur
        functions in superspace, :arXiv:`1608.08577`

AUTHORS:

- Mike Zabrocki (Sept 2017) - initial implementation
"""
from sage.structure.parent import Parent
from sage.structure.unique_representation import UniqueRepresentation
from sage.categories.rings import Rings
from sage.categories.fields import Fields
from sage.categories.graded_hopf_algebras import GradedHopfAlgebras
from sage.categories.realizations import Category_realization_of_parent
from sage.combinat.permutation import Permutation
from sage.sets.family import Family
from sage.sets.positive_integers import PositiveIntegers
from sage.misc.bindable_class import BindableClass
from sage.categories.graded_hopf_algebras_with_basis import GradedHopfAlgebrasWithBasis
from sage.rings.integer import Integer
from sage.combinat.ncsf_qsym.generic_basis_code import AlgebraMorphism
#from sage.combinat.superpartition import SuperPartition as SP

class SFSuperSpaceAlgebraMorphism(AlgebraMorphism):
    r"""
    An algebra morphism on symmetric functions in super space

    The generators are indexed by integers, non-positive integers are
    indices of the fermionic generators and the positive integers
    are mapped to the bosonic generators.
    """
    def super_partition_to_generators(self, sp):
        r"""
        A list of integers representing the generators from a multiplicative basis
        """
        return [-a for a in sp[0]]+[a for a in sp[1]]

    def _on_basis(self, sp):
        r"""
        Computes the image of this morphism on the basis element indexed by
        ``sp``.

        INPUT:

        - ``sp`` -- a SuperPartition

        OUTPUT:

        - element of the codomain

        EXAMPLES::

            sage: 
            sage: SFSS = SymmetricFunctionsinSuperSpace(QQ)
            sage: p = SFSS.p()
            sage: h = SFSS.h()
            sage: f = SFSuperSpaceAlgebraMorphism(p, lambda i : h[[],[abs(i)]], codomain=h)
            sage: f._on_basis([[1,0],[3,2]])
            h[; 3, 2, 1]
            sage: f = SFSuperSpaceAlgebraMorphism(p, lambda i : h[[abs(i)],[]], codomain=h)
            sage: f._on_basis([[0],[3, 1]])
            h[3, 1, 0; ]
            sage: f._on_basis([[1],[1]])
            0
        """
        if self._anti:
            c = reversed(self.super_partition_to_generators(sp))
        else:
            c = self.super_partition_to_generators(sp)
        return self.codomain().prod(self._on_generators(i) for i in c)

#from sage.combinat.sf import SymmetricFunctions
def valid_fermionic_matchings(sp1, sp2, a_sp3):
    r"""
    Generator for ways of matching the fermionic parts.

    For a fixed fermionic part ``a_sp3`` such that the length is equal to the
    sums of the lengths of the fermionic parts of ``sp1`` and ``sp2``, find
    all ways of covering ``a_sp3`` with the fermionic parts of ``sp1`` and ``sp2``.
    The output of this function will be a list of matchings, where a matching
    is a list of 4 objects:
    - a permutation of a subset of the parts of ``a_sp3``

    OUTPUT:

    - a list

    EXAMPLES::

        sage: #from sage.combinat.????.sfss import valid_fermionic_matchings
        sage: [p for p in valid_fermionic_matchings([[1,0],[1]], [[0],[2,1,1]], [2,1,0])]
        [[[1, 2], [0, 2], [0], [0]],
         [[2, 1], [1, 1], [0], [0]],
         [[2, 0], [1, 0], [1], [1]]]
        sage: [p for p in valid_fermionic_matchings([[1,0],[1]], [[0],[2,1,1]], [3,1,0])]
        [[[3, 1], [2, 1], [0], [0]], [[3, 0], [2, 0], [1], [1]]]
    """
    for SS in Subsets(a_sp3,len(sp1[0])): #pick a subset of fermionic part of a_sp3
        for pi in Permutations(SS): #rearrange them in all possible ways
            vs = [pi[i]-sp1[0][i] for i in range(len(sp1[0]))]
            if min(vs+[0])>=0 and all(vs.count(v)<=sp2[1].count(v) for v in vs if v>0):
                for tau in Permutations(Set(a_sp3).difference(SS)):
                    ws = [tau[i]-sp2[0][i] for i in range(len(sp2[0]))]
                    if min(ws+[0])>=0 and all(ws.count(w)<=sp1[1].count(w) for w in ws if w>0):
                        yield [pi,vs,tau,ws]

def remove_parts(la, SS):
    r"""
    Remove multiple parts from a partition.

    OUTPUT:

    - a list of integers

    EXAMPLES::

        sage: #from sage.combinat.????.sfss import remove_parts
        sage: remove_parts([4,4,3,3,3,2,2,2,1,1],[3,3,2,1,1])
        [4, 4, 3, 2, 2]
    """
    return sum(([k]*(la.count(k)-SS.count(k)) for k in uniq(la)[::-1]),[])

def m_prod_fixed_a_match(SFSS_m, sp1, sp2, a_sp3, mt):
    r"""
    The terms with fixed fermionic part and matching in a monomial product.

    The matching ``mt`` is a list consisting of permutations of the
    parts of ``a_sp3`` and lists indicating which bosonic parts will
    be used from ``sp1`` and ``sp2`` in the parts of ``a_sp3``.  There is
    one term in the sum for each term in the product of the remaining bosonic
    parts of ``sp1`` and ``sp2`` considered as monomial symmetric functions.

    INPUT:
    
    - ``SFSS_m`` -- a monomial basis
    - ``sp1, sp2`` -- super partitions
    - ``a_sp3`` -- a strict partition
    - ``mt`` -- a list

    OUTPUT:

    - an element in the ``SFSS_m`` basis

    Below we have calculated the terms in example (2.41) and (2.42) from
    [DLM06]_ in detail.

    EXAMPLES::

        sage: m = SymmetricFunctionsinSuperSpace(QQ).m()
        sage: m_prod_fixed_a_match(m,[[1,0],[1]],[[0],[2,1,1]],[2,1,0],[[1,2],[0,2],[0],[0]])
        3*m[2, 1, 0; 1, 1, 1] + m[2, 1, 0; 2, 1]
        sage: m_prod_fixed_a_match(m,[[1,0],[1]],[[0],[2,1,1]],[2,1,0],[[2,1],[1,1],[0],[0]])
        m[2, 1, 0; 2, 1] + m[2, 1, 0; 3]
        sage: m_prod_fixed_a_match(m,[[1,0],[1]],[[0],[2,1,1]],[2,1,0],[[2,0],[1,0],[1],[1]])
        m[2, 1, 0; 2, 1]
        sage: m_prod_fixed_a_match(m,[[1,0],[1]],[[0],[2,1,1]],[3,1,0],[[3,1],[2,1],[0],[0]])
        2*m[3, 1, 0; 1, 1] + m[3, 1, 0; 2]
        sage: m_prod_fixed_a_match(m,[[1,0],[1]],[[0],[2,1,1]],[3,1,0],[[3,0],[2,0],[1],[1]])
        m[3, 1, 0; 1, 1]
    """
    m = SFSS_m._Sym_m
    ga = remove_parts(sp2[1], mt[1])
    nu = remove_parts(sp1[1], mt[3])
    return SFSS_m.sum(SFSS_m.term(SuperPartition([a_sp3,list(la)]),c) \
        for (la,c) in m(ga)*m(nu))

def sign_matching(sp1, sp2, mt):
    tp = list(mt[0])+list(mt[2])
    prm2 = Permutation([j for (i,j) in sorted([(tp[-k-1],k+1) for k in range(len(tp))])])
    return prm2.sign()

def m_m_mult_fixed_a(SFSS_m, sp1, sp2, a_sp3):
    if len(sp1[0])+len(sp2[0])!=len(a_sp3):
        return SFSS_m.zero()
    if sum(sp1[0])+sum(sp2[0])>sum(a_sp3):
        return SFSS_m.zero()
    return SFSS_m.sum(sign_matching(sp1, sp2, mt)*m_prod_fixed_a_match(SFSS_m, sp1, sp2, a_sp3, mt)
            for mt in valid_fermionic_matchings(sp1, sp2, a_sp3))

class SymmetricFunctionsinSuperSpace(UniqueRepresentation, Parent):
    def __init__(self, R):
        r"""
        TESTS::

            sage: SFSS1 = SymmetricFunctionsinSuperSpace(FiniteField(23))
            sage: SFSS2 = SymmetricFunctionsinSuperSpace(Integers(23))
            sage: TestSuite(SymmetricFunctionsinSuperSpace(QQ)).run()
        """
        assert(R in Fields() or R in Rings())
        self._base = R # Won't be needed once CategoryObject won't override base_ring
        Parent.__init__(self, category = GradedHopfAlgebras(R).WithRealizations())
        h = self.Complete()
        e = self.Elementary()
        p = self.Power()
        m = self.Monomial()
        s = self.Schur()
        ss = self.Schur_s()
        sb = self.Schur_b()
        ssb = self.Schur_sb()

        # coercions between multiplicative bases
        for b1 in ['e','h','p']:
            for b2 in ['e','h','p']:
                if b1!=b2:
                    eval(b1+'.algebra_morphism('+b2+'.'+b1+\
                        '_to_self_on_generator, codomain = '\
                        +b2+').register_as_coercion()')
        p.module_morphism(m.power_to_self_on_basis, codomain = m).register_as_coercion()
        m.module_morphism(function=p.monomial_to_self_by_triangularity, codomain = p).register_as_coercion()
        ss.module_morphism(ss.self_to_complete_on_basis, codomain = h).register_as_coercion()
        h.module_morphism(function=ss.complete_to_self_by_triangularity, codomain = ss).register_as_coercion()
        ssb.module_morphism(ssb.self_to_complete_on_basis, codomain = h).register_as_coercion()
        h.module_morphism(function=ssb.complete_to_self_by_triangularity, codomain = ssb).register_as_coercion()
        sb.module_morphism(sb.self_to_elementary_on_basis, codomain = e).register_as_coercion()
        e.module_morphism(function=sb.elementary_to_self, codomain = sb).register_as_coercion()
        s.module_morphism(s.self_to_elementary_on_basis, codomain = e).register_as_coercion()
        e.module_morphism(function=s.elementary_to_self, codomain = s).register_as_coercion()

    def a_realization(self):
        r"""
        A basis of the symmetric functions in super space.

        This particular realization is the Power basis.

        EXAMPLES::

            sage: SymmetricFunctionsinSuperSpace(QQ).a_realization()
        """
        return self.Power()

    def _repr_(self): # could be taken care of by the category
        r"""
        EXAMPLES::

            sage: SFSS = SymmetricFunctionsinSuperSpace(ZZ)
            sage: SFSS._repr_()
            'Symmetric Functions in super space over the Integer Ring'
        """
        return "Symmetric Functions in super space over the %s"%self.base_ring()

    _shorthands = tuple(['e', 'h', 'p', 'm', 'ss', 'sb', 'ssb', 's'])

    class Bases(Category_realization_of_parent):
        """
        Category of bases of symmetric functions in super space.

        EXAMPLES::

            sage: SFSS = SymmetricFunctionsinSuperSpace(QQ)
            sage: SFSS.Bases()
            Category of bases of Symmetric Functions in super space over the Rational Field
            sage: p = SFSS.Power()
            sage: p in SFSS.Bases()
            True
        """
        def super_categories(self):
            r"""
            Return the super categories of the category of bases of
            symmetric functions in super space.

            OUTPUT:

            - a list

            TESTS::

                sage: SFSS = SymmetricFunctionsinSuperSpace(QQ)
                sage: SFSS.Bases().super_categories()
                [Category of realizations of Symmetric Functions in super space over the Rational Field,
                 Category of graded hopf algebras with basis over Rational Field,
                 Join of Category of realizations of hopf algebras over Rational Field and Category of graded algebras over Rational Field]
            """
            R = self.base().base_ring()
            #from sage.combinat.ncsf_qsym.generic_basis_code import GradedModulesWithInternalProduct
            # if we can implement an internal product add this category
            return [self.base().Realizations(), GradedHopfAlgebrasWithBasis(R), \
                GradedHopfAlgebras(R).Realizations()]

        class ParentMethods:
            def one_basis(self):
                r"""
                The empty super-partition.

                OUTPUT:

                - The empty super-partition.

                EXAMPLES::

                    sage: p=SymmetricFunctionsinSuperSpace(QQ).Power()
                    sage: parent(p)
                    <class '__main__.SymmetricFunctionsinSuperSpace.Power_with_category'>
                    sage: parent(p).one_basis()
                    [; ]
                    sage: p.one()
                    p[; ]
                """
                return SuperPartitions()([[],[]])

            def __getitem__(self, c, *rest):
                r"""
                This method implements the abuses of notations::

                    sage: p = SymmetricFunctionsinSuperSpace(QQ).Power()
                    sage: p[[2],[1]]
                    p[2; 1]
                    sage: p[[[2],[1]]]
                    p[2; 1]
                    sage: p[SuperPartition([[2],[1]])]
                    p[2; 1]
                    sage: p[-2, 1, 0]
                    p[-2, 0, 1]
                """
                if isinstance(c, SuperPartition):
                    assert len(rest) == 0
                else:
                    if isinstance(c, (int, Integer)):
                        c = self._indices([c])
                    elif list(c) in self._indices:
                        c = self._indices(list(c))
                    else:
                        raise ValueError("%s must be a %s"%(c,self._indices))
                return self.monomial(c)

            def _repr_(self):
                r"""
                TESTS::

                    sage: h = SymmetricFunctionsinSuperSpace(QQ).Complete()
                    sage: h._repr_()
                    'Symmetric Functions in super space over the Rational Field in the Complete basis'
                """
                return "%s in the %s basis" % (self.realization_of(), self._realization_name())

            def degree_on_basis(self, sp):
                r"""
                The degree of a basis element is the degree of the super partition.

                EXAMPLES::
                    sage: h = SymmetricFunctionsinSuperSpace(QQ).Complete()
                    sage: h.degree_on_basis(SuperPartition([[2,1],[3,1,1]]))
                    8
                    sage: h.degree_on_basis(SuperPartition([[],[]]))
                    0
                """
                return sp.degree()

            def bi_degree_on_basis(self, sp):
                r"""
                The bi-degree of the super partition is the bosonic and fermionic degree.

                EXAMPLES::

                    sage: h = SymmetricFunctionsinSuperSpace(QQ).Complete()
                    sage: h.bi_degree_on_basis(SuperPartition([[2,1],[3,1,1]]))
                    (8, 2)
                    sage: h.bi_degree_on_basis(SuperPartition([[],[]]))
                    (0, 0)
                """
                return sp.bi_degree()

        class ElementMethods:
            def is_homogeneous_bi_degree(self):
                r"""
                Test if super partitions an element all have the same bi-degree.

                EXAMPLES::

                    sage: h = SymmetricFunctionsinSuperSpace(QQ).Complete()
                    sage: (h[[1],[2,1]]+h[[1,0],[3]]).is_homogeneous_bi_degree()
                    False
                    sage: (h[[1],[2,1]]+h[[0],[4]]).is_homogeneous_bi_degree()
                    True
                """
                bi_degree_on_basis = self.parent().bi_degree_on_basis
                bi_degree = None
                for m in self.support():
                    if bi_degree is None:
                        bi_degree = bi_degree_on_basis(m)
                    else:
                        if bi_degree != bi_degree_on_basis(m):
                            return False
                return True

            def bi_degree(self):
                r"""
                The bi-degree element if the element is homogeneous bi_degree.

                EXAMPLES::

                    sage: h = SymmetricFunctionsinSuperSpace(QQ).Complete()
                    sage: (h[[1],[2,1]]+h[[1,0],[3]]).bi_degree()
                    ...
                    ValueError: element is not of homogeneous bi-degree
                    sage: (h[[1],[2,1]]+h[[0],[4]]).bi_degree()
                    (4, 1)
                """
                if not self.support():
                    raise ValueError("the zero element does not have a well-defined bi-degree")
                if not self.is_homogeneous_bi_degree():
                    raise ValueError("element is not of homogeneous bi-degree")
                return self.parent().bi_degree_on_basis(self.leading_support())

            def maximal_bi_degree(self):
                r"""
                The maximal bi-degree over the support of an element.

                EXAMPLES::

                    sage: h = SymmetricFunctionsinSuperSpace(QQ).Complete()
                    sage: (h[[1],[2,1]]+h[[1,0],[3]]).maximal_bi_degree()
                    (4, 2)
                    sage: (h[[1],[2,1]]+h[[0],[4]]).maximal_bi_degree()
                    (4, 1)
                """
                if self.is_zero():
                    raise ValueError("the zero element does not have a well-defined degree")
                bi_degree_on_basis = self.parent().bi_degree_on_basis
                return max(bi_degree_on_basis(m) for m in self.support())

            def all_bi_degrees(self):
                r"""
                A list of all the bi-degrees in the support of the element

                OUTPUT:

                - a list of tuples of integers

                EXAMPLES::

                    sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                    sage: (h[0,3,2]+h[-1,1]).all_bi_degrees()
                    [(2, 1), (5, 1)]
                    sage: (h[[]]+h[0]+h[-1]).all_bi_degrees()
                    [(0, 0), (0, 1), (1, 1)]
                """
                bi_degree_on_basis = self.parent().bi_degree_on_basis
                return uniq(bi_degree_on_basis(m) for m in self.support())

            def homogeneous_bi_degree_component(self, bd):
                r"""
                Take the component of fixed bi_degree

                INPUT:

                - ``bd`` -- a tuple of integers corresponding to a bidegree

                OUTPUT:

                - a supersymmetric function

                EXAMPLES::

                    sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                    sage: (h[1,1]+h[-1,2]+h[-2,1]).homogeneous_bi_degree_component((3,1))
                    h[1; 2] + h[2; 1]
                    sage: (h[1,1]+h[-1,2]+h[-2,1]).homogeneous_bi_degree_component((2,0))
                    h[; 1, 1]
                    sage: (h[1,1]+h[-1,2]+h[-2,1]).homogeneous_bi_degree_component((1,0))
                    0
                """
                b = self.parent()
                return b.sum(c*b(sp) for (sp,c) in self if sp.bi_degree()==bd)

            def scalar(self, other):
                r"""
                The scalar product between ``self`` and ``other``.

                The power sum elements are orthonormal.

                INPUT:

                - ``other`` -- an element of symmetric functions in super space

                OUTPUT:

                - an element of the base ring

                EXAMAPLES::

                    sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                    sage: h[[],[3,1]].scalar(h[[1],[2,1]])
                    0
                    sage: h[[1],[2,1]].scalar(h[[1],[2,1]])
                    9
                """
                p = self.parent().realization_of().Power()
                px = p(self)
                py = p(other)
                return sum(sp.zee()*c*py.coefficient(sp) for (sp,c) in px)

            def skew_by(self, other, side='right'):
                r"""
                Compute the ``skew_by`` operation by coersion to the power basis

                INPUT:

                - ``other`` - a supersymmetric function

                - ``side`` - either 'right' or 'left' (default: 'right')

                OUTPUT:

                - a supersymmetric function

                EXAMPLES::

                    sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                    sage: (h[2]+h[1]+h[[]]).skew_by(h[1]+h[[]])
                    2*h[; ] + 2*h[; 1] + h[; 2]
                    sage: (h[2]+h[1]+h[[]]).skew_by(h[2])
                    h[; ]
                    sage: (h[2]+h[1]+h[[]]).skew_by(h[0])
                    0
                    sage: (h[-1,1]+h[-1]+h[-2]).skew_by(h[0])
                    h[; 1] + h[; 1, 1] + h[; 2]
                """
                p = self.parent().realization_of().Power()
                return self.parent()(p(self).skew_by(other, side))

            def omega(self):
                r"""
                The involution omega.

                The omega is computed by coercion to the power sum basis.

                EXAMPLES::

                    sage: SFSS = SymmetricFunctionsinSuperSpace(QQ)
                    sage: (h,e) = (SFSS.h(), SFSS.e())
                    sage: all(h(sp).omega()==e(sp) for sp in SuperPartitions(3))
                    True
                    sage: e[[1,0],[2,2,1]].omega()
                    -e[1, 0; 1, 1, 1, 1, 1] + 2*e[1, 0; 2, 1, 1, 1] - e[1, 0; 2, 2, 1]
                """
                p = self.parent().realization_of().Power()
                return self.parent()(p(self).omega())

            def to_symmetric_function():
                """
                Default implementation: convert to other and do the operation there
                """
                pass

    class MultiplicativeBases(Category_realization_of_parent):
        def super_categories(self):
            r"""
            Return the super categories of the category of multiplicative
            bases of the summetric functions in super space.

            OUTPUT:

            - a list

            EXAMPLES::

                sage: SFSS = SymmetricFunctionsinSuperSpace(QQ)
                sage: SFSS.MultiplicativeBases().super_categories()
                [Category of bases of Symmetric Functions in super space over the Rational Field]
            """
            return [self.base().Bases()]

        class ParentMethods:
            def algebra_generators(self):
                r"""
                A family of generators associated to a multiplicative basis.

                The generators are ordered so that the non-positive integers
                are mapped to a fermionic generator and the positive
                integers are mapped to the symmetric generator.

                EXAMPLES::

                    sage: e = SymmetricFunctionsinSuperSpace(QQ).Elementary()
                    sage: e.algebra_generators()
                    Lazy family (pos_int_to_gen(i))_{i in Positive integers}
                    sage: it = e.algebra_generators().__iter__()
                    sage: [next(it) for i in range(6)]
                    [e[0; ], e[; 1], e[1; ], e[; 2], e[2; ], e[; 3]]
                """
                def pos_int_to_gen(n):
                    if n<=0:
                        return self.monomial(self._indices([[abs(n)],[]]))
                    else:
                        return self.monomial(self._indices([[],[n]]))
                return Family(Integers(), pos_int_to_gen)

            def algebra_morphism(self, on_generators, **keywords):
                """
                Extend a map on the generators of a multiplicative basis to the whole basis.

                Given a map defined on the generators of the multiplicative
                basis ``self``, return the algebra morphism that extends
                this map to the whole algebra of symmetric functions in super
                space.
                # FIX DOCUMENTATION
                # written for NCSF

                INPUT:

                - ``on_generators`` -- a function defined on the index set of
                  the generators (that is, on the positive integers)
                - ``codomain`` -- the codomain of ``on_generators``
                - ``anti`` -- a boolean; defaults to ``False``
                - ``category`` -- a category; defaults to ``None``

                OUTPUT:

                - The algebra morphism of ``self`` which is defined by
                  ``on_generators`` in the basis ``self``. When ``anti``
                  is set to ``True``, an algebra anti-morphism is
                  computed instead of an algebra morphism.

                EXAMPLES::

                    sage: SFSS = SymmetricFunctionsinSuperSpace(QQ)
                    sage: p = SFSS.p()
                    sage: neg = lambda i: p.algebra_generators()[-i] 
                    sage: f = p.algebra_morphism(neg, codomain = p)
                    sage: f
                    Generic endomorphism of Non-Commutative Symmetric Functions over the Rational Field in the Psi basis
                    sage: f(2*p[[1],[2]] + 3 * p[[2,1],[]] + p[[2,0],[]])
                    3*p[; 2, 1] + p[0; 2] + 2*p[2; 1]
                    sage: f.category()
                    Category of endsets of unital magmas and right modules over Rational Field and left modules over Rational Field

                When extra properties about the morphism are known, one
                can specify the category of which it is a morphism::

                    sage: alpha = lambda i: -p.algebra_generators()[i]
                    sage: f = p.algebra_morphism(alpha, codomain = p, category = GradedHopfAlgebrasWithBasis(QQ))
                    sage: f
                    Generic endomorphism of Symmetric Functions in super space over the Rational Field in the Power basis
                    sage: f(2*p[[],[1]] + 3 * p[[1],[]] + p[[0],[1]] )
                    -2*p[; 1] + p[0; 1] - 3*p[1; ]
                    sage: f.category()
                    Category of endsets of hopf algebras over Rational Field and graded modules over Rational Field

                If ``anti`` is true, this returns an anti-algebra morphism::

                    sage: f = p.algebra_morphism(alpha, codomain = p, anti=True)
                    sage: f
                    Generic endomorphism of Symmetric Functions in super space over the Rational Field in the Power basis
                    sage: f(2*p[[3,1],[]] + 3 * p[[2,1],[1]])
                    3*p[2, 1; 1] - 2*p[3, 1; ]
                    sage: f.category()
                    Category of endsets of modules with basis over Rational Field
                """
                return SFSuperSpaceAlgebraMorphism(self, on_generators, **keywords)

            def product_on_basis(self, spartition1, spartition2):
                r"""
                Product of two multiplicative elements indexed by super partitions.

                Generators from the symmetric component commute while
                generators from the antisymmetric component satisfy
                `a_i a_j = - a_j a_i`

                INPUT:

                - ``spartition1, spartition2`` -- super partitions or lists

                OUTPUT:

                - a monomial in the basis

                EXAMPLES::

                    sage: e = SymmetricFunctionsinSuperSpace(QQ).e()
                    sage: e.product_on_basis([[1,0],[1,1]],[[2],[2]])
                    e[2, 1, 0; 2, 1, 1]
                    sage: e.product_on_basis([[1],[1,1]],[[2],[2]])
                    -e[2, 1; 2, 1, 1]
                    sage: e.product_on_basis([[2],[1,1]],[[2],[2]])
                    0
                """
                pa = spartition1[0]+spartition2[0]
                pb = sorted([(pa[i],i+1) for i in range(len(pa))],reverse=True)
                sign = Permutation([v[1] for v in pb]).sign()
                ps = sorted(spartition1[1]+spartition2[1],reverse=True)
                if len(uniq(pa))==len(pa):
                    return sign*self.monomial(SuperPartition([[v[0] for v in pb], ps]))
                else:
                    return self.zero()

    class Monomial(CombinatorialFreeModule, BindableClass):
        def __init__(self, SFSS):
            r"""
            TESTS::

                sage: m = SymmetricFunctionsinSuperSpace(QQ).Monomial()
                sage: TestSuite(m).run()
            """
            CombinatorialFreeModule.__init__(self, SFSS.base_ring(), SuperPartitions(),
                                             prefix='m', bracket=False, category=SFSS.Bases())
            self._Sym_m = SymmetricFunctions(self.base_ring()).m()
            self._Sym_p = SymmetricFunctions(self.base_ring()).p()

        @cached_method
        def power_to_self_on_basis(self, sp):
            r"""
            Expand a power basis element in the monomial basis.

            INPUT:

            - ``sp`` -- a super partition

            EXAMPLES::

                sage: m = SymmetricFunctionsinSuperSpace(QQ).Monomial()
                sage: m.power_to_self_on_basis([[2,0],[1,1]])
                2*m[2, 0; 1, 1] + m[2, 0; 2] + 2*m[2, 1; 1] + 2*m[3, 0; 1] + 2*m[3, 1; ] + m[4, 0; ]
            """
            return self.sum(self.term(SuperPartition([sp[0],[]]))*\
                self.term(SuperPartition([[],list(la)]),c) \
                for (la,c) in self._Sym_m(self._Sym_p(sp[1])))

        @cached_method
        def product_on_basis(self, sp1, sp2):
            r"""
            Product of two monomial basis elements.

            The product of monomial basis elements is calculated for a
            fixed fermionic partition.

            INPUT:

            - ``sp1``, ``sp2`` -- super partitions

            EXAMPLES::

                sage: m = SymmetricFunctionsinSuperSpace(QQ).Monomial()
                sage: m.product_on_basis(SuperPartition([[1],[1]]),SuperPartition([[0],[2]]))
                m[1, 0; 2, 1] + m[1, 0; 3] + m[3, 0; 1] + m[3, 1; ]
                sage: m.product_on_basis(SuperPartition([[1],[]]),SuperPartition([[1],[]]))
                0
                sage: m.product_on_basis(SuperPartition([[0],[]]),SuperPartition([[1],[]]))
                -m[1, 0; ]
            """
            min_a_size = sum(sp1[0]+sp2[0])
            out = self.zero()
            for r in range(min_a_size, min_a_size+sum(sp1[1]+sp2[1])+1):
                pars = Partitions(r, max_slope=-1, length=len(sp1[0]+sp2[0]))
                pars2 = Partitions(r, max_slope=-1, length=len(sp1[0]+sp2[0])-1)
                out += self.sum(m_m_mult_fixed_a(self, sp1, sp2, list(a_sp3)) for a_sp3 in pars)
                out += self.sum(m_m_mult_fixed_a(self, sp1, sp2, list(a_sp3)+[0]) for a_sp3 in pars2)
            return out

    m = Monomial

    class Schur_sb(CombinatorialFreeModule, BindableClass):
        r"""
        The Schur basis arising from `q=t=0` specialization of Macdonald.
        
        The Pieri rule for this basis is described in [JL16]_ .

        EXAMPLES::

            sage: s = SymmetricFunctionsinSuperSpace(QQ).Schur_sb()
            sage: s[[1,0],[]]*s[[],[3]]
            ss[1, 0; 3]
            sage: s[[1],[2]]*s[[],[3]]   ################
            ss[0; 4, 2] + ss[1; 3, 2] + ss[1; 4, 1] + ss[1; 5]
            sage: s[[1],[2]]*ss[[3],[]]
            ss[1, 0; 5] - ss[3, 1; 2]
            sage: ss[[0],[1]].coproduct()
            ss[; ] # ss[0; 1] + ss[; 1] # ss[0; ] + ss[0; ] # ss[; 1] + ss[0; 1] # ss[; ]
            sage: ss[[1],[2]].omega()
            ss[0; 1, 1, 1] + ss[0; 2, 1] - ss[1; 1, 1]
            sage: sb = SymmetricFunctionsinSuperSpace(QQ).Schur_b()
            sage: sb(ss[[1],[2]].omega())
            sb[1; 2]
            sage: ss[[2],[1]].antipode()
            2*ss[0; 1, 1, 1] + 2*ss[0; 2, 1] - 2*ss[1; 1, 1] - 2*ss[1; 2] + ss[2; 1]
        """
        def __init__(self, SFSS):
            r"""
            TESTS::

                sage: ss = SymmetricFunctionsinSuperSpace(QQ).Schur_s()
                sage: TestSuite(ss).run()
            """
            CombinatorialFreeModule.__init__(self, SFSS.base_ring(), \
                SuperPartitions(), prefix='ssb', bracket=False, \
                category=SFSS.Bases())
            self._h = SFSS.Complete()

        @cached_method
        def self_to_complete_on_basis(self, sp):
            r"""
            Convert the Schur basis to complete basis using the Pieri rule.

            INPUT:

            - ``sp`` -- a super-partition

            OUTPUT:

            - an expression in the complete basis

            EXAMPLES::

                sage: ss = SymmetricFunctionsinSuperSpace(QQ).Schur_s()
                sage: ss.self_to_complete_on_basis(SuperPartition([[1],[1]]))
                -h[0; 2] + h[1; 1]
                sage: ss.self_to_complete_on_basis(SuperPartition([[1],[1,1]]))
                -h[0; 2, 1] + h[0; 3] + h[1; 1, 1] - h[1; 2]
            """
            if SuperPartition(sp).bosonic_length()>0:
                sp2 = SuperPartition([sp[0],sp[1][1:]])
                return self.self_to_complete_on_basis(sp2)*self._h([[],[sp[1][0]]])\
                    - sum(self.self_to_complete_on_basis(spa) for spa in \
                          theorem_10(sp2, sp[1][0]) if spa!=sp)
            else:
                return self._h.prod(self._h([[a],[]]) for a in sp[0])
    
        def complete_to_self_by_triangularity(self, h_expr):
            r"""
            Convert an expression in the complete basis to the Schur basis

            INPUT:

            - ``h_expr`` - an expression in the complete basis

            OUTPUT:

            - an expression in the Schur-* basis

            EXAMPLES::

                sage: ss = SymmetricFunctionsinSuperSpace(QQ).Schur_s()
                sage: h = SymmetricFunctionsinSuperSpace(QQ).Complete()
                sage: ss.complete_to_self_by_triangularity(-h[[0],[2]]+h[[1],[1]])
                ss[1; 1]
                sage: ss.self_to_complete_on_basis(SuperPartition([[1],[1,1]]))
                -h[0; 2, 1] + h[0; 3] + h[1; 1, 1] - h[1; 2]
                sage: ss(ss.self_to_complete_on_basis(SuperPartition([[1],[1,1]])))
                ss[1; 1, 1]
            """
            out = self.zero()
            while h_expr!=0:
                (a, sp, c) = min([[-sum(sp[0]),sp,c] for (sp,c) in h_expr])
                htrm = self.self_to_complete_on_basis(sp)
                h_expr -= c*htrm.coefficient(sp)*htrm
                out += self.term(sp, c*htrm.coefficient(sp))
            return out

    ssb = Schur_sb

    class Schur_s(CombinatorialFreeModule, BindableClass):
        r"""
        The Schur-* basis arising from `q=t=0` specialization of Macdonald.

        The basis is calculated by the Pieri rule in order to obtain the
        the expansion in the complete basis.  It is calculated from the
        complete basis by identifying a triangularity relation.
        
        The Pieri rule for this basis is described in [JL16]_ .

        EXAMPLES::

            sage: ss = SymmetricFunctionsinSuperSpace(QQ).Schur_s()
            sage: ss[[1,0],[]]*ss[[],[3]]
            ss[1, 0; 3]
            sage: ss[[1],[2]]*ss[[],[3]]
            ss[0; 4, 2] + ss[1; 3, 2] + ss[1; 4, 1] + ss[1; 5]
            sage: ss[[1],[2]]*ss[[3],[]]
            ss[1, 0; 5] - ss[3, 1; 2]
            sage: ss[[0],[1]].coproduct()
            ss[; ] # ss[0; 1] + ss[; 1] # ss[0; ] + ss[0; ] # ss[; 1] + ss[0; 1] # ss[; ]
            sage: ss[[1],[2]].omega()
            ss[0; 1, 1, 1] + ss[0; 2, 1] - ss[1; 1, 1]
            sage: sb = SymmetricFunctionsinSuperSpace(QQ).Schur_b()
            sage: sb(ss[[1],[2]].omega())
            sb[1; 2]
            sage: ss[[2],[1]].antipode()
            2*ss[0; 1, 1, 1] + 2*ss[0; 2, 1] - 2*ss[1; 1, 1] - 2*ss[1; 2] + ss[2; 1]
        """
        def __init__(self, SFSS):
            r"""
            TESTS::

                sage: ss = SymmetricFunctionsinSuperSpace(QQ).Schur_s()
                sage: TestSuite(ss).run()
            """
            CombinatorialFreeModule.__init__(self, SFSS.base_ring(), \
                SuperPartitions(), prefix='ss', bracket=False, \
                category=SFSS.Bases())
            self._h = SFSS.Complete()

        @cached_method
        def self_to_complete_on_basis(self, sp):
            r"""
            Convert the Schur-* basis to complete basis using the Pieri rule.

            INPUT:

            - ``sp`` -- a super-partition

            OUTPUT:

            - an expression in the complete basis

            EXAMPLES::

                sage: ss = SymmetricFunctionsinSuperSpace(QQ).Schur_s()
                sage: ss.self_to_complete_on_basis(SuperPartition([[1],[1]]))
                -h[0; 2] + h[1; 1]
                sage: ss.self_to_complete_on_basis(SuperPartition([[1],[1,1]]))
                -h[0; 2, 1] + h[0; 3] + h[1; 1, 1] - h[1; 2]
            """
            if SuperPartition(sp).bosonic_length()>0:
                sp2 = SuperPartition([sp[0],sp[1][1:]])
                return self.self_to_complete_on_basis(sp2)*self._h([[],[sp[1][0]]])\
                    - sum(self.self_to_complete_on_basis(spa) for spa in \
                          sp2.add_horizontal_border_strip(sp[1][0]) if spa!=sp)
            else:
                return self._h.prod(self._h([[a],[]]) for a in sp[0])
    
        def complete_to_self_by_triangularity(self, h_expr):
            r"""
            Convert an expression in the complete basis to the Schur-* basis

            INPUT:

            - ``h_expr`` - an expression in the complete basis

            OUTPUT:

            - an expression in the Schur-* basis

            EXAMPLES::

                sage: ss = SymmetricFunctionsinSuperSpace(QQ).Schur_s()
                sage: h = SymmetricFunctionsinSuperSpace(QQ).Complete()
                sage: ss.complete_to_self_by_triangularity(-h[[0],[2]]+h[[1],[1]])
                ss[1; 1]
                sage: ss.self_to_complete_on_basis(SuperPartition([[1],[1,1]]))
                -h[0; 2, 1] + h[0; 3] + h[1; 1, 1] - h[1; 2]
                sage: ss(ss.self_to_complete_on_basis(SuperPartition([[1],[1,1]])))
                ss[1; 1, 1]
            """
            out = self.zero()
            while h_expr!=0:
                (a, sp, c) = min([[-sum(sp[0]),sp,c] for (sp,c) in h_expr])
                htrm = self.self_to_complete_on_basis(sp)
                h_expr -= c*htrm.coefficient(sp)*htrm
                out += self.term(sp, c*htrm.coefficient(sp))
            return out

    ss = Schur_s

    class Schur_b(CombinatorialFreeModule, BindableClass):
        r"""
        The Schur-bar basis as the `q=t=\infty` specialization of Macdonald.

        The basis is calculated from the complete expansion of the Schur-star
        basis and then the same coefficients are used for the elementary
        basis for the Schur-bar.  It is calculated from the
        elementary basis by identifying a triangularity relation.

        This basis is dual to the one-true Schur basis.
        It is also the involution omega applied to the Schur-* basis.
        
        The Pieri rule for this basis is described in [JL16]_ .

        EXAMPLES::

            sage: sb = SymmetricFunctionsinSuperSpace(QQ).Schur_b()
            sage: sb[[1,0],[]]*sb[[],[3]]
        """
        def __init__(self, SFSS):
            r"""
            TESTS::

                sage: sb = SymmetricFunctionsinSuperSpace(QQ).Schur_b()
                sage: TestSuite(sb).run()
            """
            CombinatorialFreeModule.__init__(self, SFSS.base_ring(), \
                SuperPartitions(), prefix='sb', bracket=False, \
                category=SFSS.Bases())
            self._e = SFSS.Elementary()
            self._ss = SFSS.Schur_s()

        @cached_method
        def self_to_elementary_on_basis(self, sp):
            r"""
            Convert the Schur-bar basis to elementary basis using omega.

            INPUT:

            - ``sp`` -- a super-partition

            OUTPUT:

            - an expression in the elementary basis

            EXAMPLES::

                sage: sb = SymmetricFunctionsinSuperSpace(QQ).Schur_b()
                sage: sb.self_to_elementary_on_basis(SuperPartition([[1],[1]]))
                -e[0; 2] + e[1; 1]
                sage: sb.self_to_elementary_on_basis(SuperPartition([[1],[1,1]]))
                -e[0; 2, 1] + e[0; 3] + e[1; 1, 1] - e[1; 2]
            """
            mc2 = binomial(sp.fermionic_degree(),2)
            return (-1)**mc2*self._e.sum(self._e.term(sp,c) for (sp,c) in \
                self._ss.self_to_complete_on_basis(sp.conjugate()))
            
    
        def elementary_to_self(self, e_expr):
            r"""
            Convert an expression in the elementary basis to the Schur-bar basis

            The conversion applies ``omega`` to the expression, converts to the
            Schur-star basis and then applies the relation that ``omega``
            applied to a Schur-star is a Schur-bar up to sign.

            INPUT:

            - ``e_expr`` - an expression in the elementary basis

            OUTPUT:

            - an expression in the Schur-bar basis

            EXAMPLES::

                sage: sb = SymmetricFunctionsinSuperSpace(QQ).Schur_b()
                sage: e = SymmetricFunctionsinSuperSpace(QQ).Elementary()
                sage: sb.elementary_to_self(-e[[0],[2]]+e[[1],[1]])
                sb[1; 1]
                sage: sb.self_to_elementary_on_basis(SuperPartition([[1],[1,1]]))
                -e[0; 2, 1] + e[0; 3] + e[1; 1, 1] - e[1; 2]
            """
            ss_e_expr = self._ss(e_expr.omega())
            return self.sum_of_terms((sp.conjugate(), \
                (-1)**binomial(sp.fermionic_sector(),2)*c) \
                for (sp,c) in ss_e_expr)

    sb = Schur_b

    class Schur(CombinatorialFreeModule, BindableClass):
        r"""
        The Schur-bar basis as the `q=t=\infty` specialization of Macdonald.

        The basis is calculated from the complete expansion of the Schur-star-bar
        basis and then the same coefficients are used for the elementary
        basis for the Schur.  It is calculated from the
        elementary basis by identifying a triangularity relation.

        This basis is the one-true Schur basis.
        It is also the involution omega applied to the Schur-star-bar basis.
        
        The Pieri rule for this basis is described in [JL16]_ .

        EXAMPLES::

            sage: sb = SymmetricFunctionsinSuperSpace(QQ).Schur_b()
            sage: sb[[1,0],[]]*sb[[],[3]]
        """
        def __init__(self, SFSS):
            r"""
            TESTS::

                sage: s = SymmetricFunctionsinSuperSpace(QQ).Schur()
                sage: TestSuite(s).run()
            """
            CombinatorialFreeModule.__init__(self, SFSS.base_ring(), \
                SuperPartitions(), prefix='s', bracket=False, \
                category=SFSS.Bases())
            self._e = SFSS.Elementary()
            self._ssb = SFSS.Schur_sb()

        @cached_method
        def self_to_elementary_on_basis(self, sp):
            r"""
            Convert the Schur basis to elementary basis using omega.

            INPUT:

            - ``sp`` -- a super-partition

            OUTPUT:

            - an expression in the elementary basis

            EXAMPLES::

                sage: s = SymmetricFunctionsinSuperSpace(QQ).Schur_b()
                sage: s.self_to_elementary_on_basis(SuperPartition([[1],[1]]))
                -e[0; 2] + e[1; 1]
                sage: s.self_to_elementary_on_basis(SuperPartition([[1],[1,1]]))
                -e[0; 2, 1] + e[0; 3] + e[1; 1, 1] - e[1; 2]
            """
            mc2 = binomial(sp.fermionic_degree(),2)
            return (-1)**mc2*self._e.sum(self._e.term(sp,c) for (sp,c) in \
                self._ssb.self_to_complete_on_basis(sp.conjugate()))
            
    
        def elementary_to_self(self, e_expr):
            r"""
            Convert an expression in the elementary basis to the Schur basis

            The conversion applies ``omega`` to the expression, converts to the
            Schur-star-bar basis and then applies the relation that ``omega``
            applied to a Schur-star-bar is a Schur up to sign.

            INPUT:

            - ``e_expr`` - an expression in the elementary basis

            OUTPUT:

            - an expression in the Schur basis

            EXAMPLES::

                sage: sb = SymmetricFunctionsinSuperSpace(QQ).Schur_b()
                sage: e = SymmetricFunctionsinSuperSpace(QQ).Elementary()
                sage: sb.elementary_to_self(-e[[0],[2]]+e[[1],[1]])
                sb[1; 1]
                sage: sb.self_to_elementary_on_basis(SuperPartition([[1],[1,1]]))
                -e[0; 2, 1] + e[0; 3] + e[1; 1, 1] - e[1; 2]
            """
            ssb_e_expr = self._ssb(e_expr.omega())
            return self.sum_of_terms((sp.conjugate(), \
                (-1)**binomial(sp.fermionic_sector(),2)*c) \
                for (sp,c) in ssb_e_expr)

    s = Schur

    class Elementary(CombinatorialFreeModule, BindableClass):
        def __init__(self, SFSS):
            r"""
            TESTS::

                sage: e = SymmetricFunctionsinSuperSpace(QQ).Elementary()
                sage: TestSuite(e).run()
            """
            CombinatorialFreeModule.__init__(self, SFSS.base_ring(), \
                SuperPartitions(), prefix='e', bracket=False, \
                category=SFSS.MultiplicativeBases())

        def p_to_self_on_generator(self, n):
            r"""
            A Power generator in the Complete basis.

            INPUT:

            - ``n`` -- an integer

            The generators are ordered so that the non-positive integers
            are mapped to a fermionic generator and the positive
            integers are mapped to the symmetric generator.

            EXAMPLES::
            
                sage: e = SymmetricFunctionsinSuperSpace(QQ).e()
                sage: e.p_to_self_on_generator(2)
                e[; 1, 1] - 2*e[; 2]
                sage: e.p_to_self_on_generator(-2)
                e[0; 1, 1] - e[0; 2] - e[1; 1] + e[2; ]
            """
            if n<=0:
                return self.agen_p_to_self(abs(n))
            else:
                return self.sgen_p_to_self(n)

        @cached_method
        def sgen_p_to_self(self, n):
            r"""
            A symmetric Power generator in the Elementary basis.

            INPUT:

            - ``n`` -- a positive integer

            EXAMPLES::
            
                sage: e = SymmetricFunctionsinSuperSpace(QQ).e()
                sage: e.sgen_p_to_self(4)
                e[; 1, 1, 1, 1] - 4*e[; 2, 1, 1] + 2*e[; 2, 2] + 4*e[; 3, 1] - 4*e[; 4]
            """
            n = Integer(n)
            if n==0:
                return self.term(SuperPartition([[],[]]))
            return self.term(SuperPartition([[],[n]]),(-1)**(n+1)*n) - \
                self.sum(self.sgen_p_to_self(r)*self.term(SuperPartition([[],[n-r]]),\
                    (-1)**(n+r)) for r in range(1,n))

        @cached_method
        def agen_p_to_self(self, n):
            r"""
            A fermionic Power generator in the Elementary basis.

            INPUT:

            - ``n`` -- a non-negative integer

            This is equation (3.44) of [DLM06]_.

            EXAMPLES::
            
                sage: e = SymmetricFunctionsinSuperSpace(QQ).e()
                sage: e.agen_p_to_self(3)
                e[0; 1, 1, 1] - 2*e[0; 2, 1] + e[0; 3] - e[1; 1, 1] + e[1; 2] + e[2; 1] - e[3; ]
            """
            n = Integer(n)
            if n==0:
                return self.term(SuperPartition([[0],[]]))
            return (-1)**n*self.term(SuperPartition([[n],[]])) + \
               self.sum(self.sgen_p_to_self(r)*self.term(\
                   SuperPartition([[n-r],[]]),(-1)**(r+n)/(n+1)) \
                   for r in range(1,n+1)) - \
               self.sum(self.agen_p_to_self(r)*self.term(\
                   SuperPartition([[],[n-r]]),(-1)**(n+r)*(r+1)/(n+1)) \
                   for r in range(n))

        def h_to_self_on_generator(self, n):
            r"""
            A Complete generator in the Elementary basis.

            INPUT:

            - ``n`` -- an integer

            The generators are ordered so that the non-positive integers
            are mapped to a fermionic generator and the positive
            integers are mapped to the symmetric generator.

            EXAMPLES::
            
                sage: e = SymmetricFunctionsinSuperSpace(QQ).e()
                sage: e.h_to_self_on_generator(2)
                e[; 1, 1] - e[; 2]
                sage: e.h_to_self_on_generator(-2)
                3*e[0; 1, 1] - 2*e[0; 2] - 2*e[1; 1] + e[2; ]
            """
            if n<=0:
                return self.agen_h_to_self(abs(n))
            else:
                return self.sgen_h_to_self(n)

        @cached_method
        def sgen_h_to_self(self, n):
            r"""
            A symmetric Complete generator expanded in the Elementary basis.

            INPUT:

            - ``n`` -- a positive integer

            EXAMPLES::

                sage: e = SymmetricFunctionsinSuperSpace(QQ).e()
                sage: e.sgen_h_to_self(4)
                e[; 1, 1, 1, 1] - 3*e[; 2, 1, 1] + e[; 2, 2] + 2*e[; 3, 1] - e[; 4]
            """
            if n==0:
                return self.term(SuperPartition([[],[]]))
            return self.sum(self.term(SuperPartition([[],la.to_list()]), \
                (-1)**(n-len(la))*factorial(len(la))/mul([factorial(a) \
                    for a in la.to_exp()])) for la in Partitions(n))

        @cached_method
        def agen_h_to_self(self, n):
            r"""
            A fermionic Complete generator expanded in the Elementary basis.

            INPUT:

            - ``n`` -- a non-negative integer

            This is equation (3.20) of [DLM06]_.

            EXAMPLES::

                sage: e = SymmetricFunctionsinSuperSpace(QQ).e()
                sage: e.agen_h_to_self(2)
                3*e[0; 1, 1] - 2*e[0; 2] - 2*e[1; 1] + e[2; ]
            """
            if n==0:
                return self.term(SuperPartition([[0],[]]))
            return -self.sum((-1)**r*self.term(SuperPartition([[],[r]]))*\
                self.agen_h_to_self(n-r) for r in range(1,n+1)) +\
                self.sum((-1)**r*self.term(SuperPartition([[r],[]]))*\
                self.sgen_h_to_self(n-r) for r in range(n+1))

    e = Elementary

    class Complete(CombinatorialFreeModule, BindableClass):
        def __init__(self, SFSS):
            r"""
            TESTS::

                sage: h = SymmetricFunctionsinSuperSpace(QQ).Complete()
                sage: TestSuite(h).run()
            """
            CombinatorialFreeModule.__init__(self, \
                SFSS.base_ring(), SuperPartitions(), prefix='h', \
                bracket=False, category=SFSS.MultiplicativeBases())

        def p_to_self_on_generator(self, n):
            r"""
            A Power generator in the Complete basis.

            INPUT:

            - ``n`` -- an integer

            The generators are ordered so that the non-positive integers
            are mapped to a fermionic generator and the positive
            integers are mapped to the symmetric generator.

            EXAMPLES::
            
                sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                sage: h.p_to_self_on_generator(2)
                -h[; 1, 1] + 2*h[; 2]
                sage: h.p_to_self_on_generator(-2)
                h[0; 1, 1] - h[0; 2] - h[1; 1] + h[2; ]
            """
            if n<=0:
                return self.agen_p_to_self(abs(n))
            else:
                return self.sgen_p_to_self(n)

        @cached_method
        def sgen_p_to_self(self, n):
            r"""
            A symmetric Power generator in the Complete basis.

            INPUT:

            - ``n`` -- a positive integer

            EXAMPLES::
            
                sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                sage: h.sgen_p_to_self(4)
                -h[; 1, 1, 1, 1] + 4*h[; 2, 1, 1] - 2*h[; 2, 2] - 4*h[; 3, 1] + 4*h[; 4]
            """
            n = Integer(n)
            if n==0:
                return self.term(SuperPartition([[],[]]))
            return self.term(SuperPartition([[],[n]]),n) - \
                self.sum(self.sgen_p_to_self(r)*self.term(SuperPartition([[],[n-r]])) \
                    for r in range(1,n))

        @cached_method
        def agen_p_to_self(self, n):
            r"""
            A fermionic Power generator in the Complete basis.

            INPUT:

            - ``n`` -- a non-negative integer

            This is equation (3.43) of [DLM06]_.

            EXAMPLES::
            
                sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                sage: h.agen_p_to_self(3)
                -h[0; 1, 1, 1] + 2*h[0; 2, 1] - h[0; 3] + h[1; 1, 1] - h[1; 2] - h[2; 1] + h[3; ]
            """
            n = Integer(n)
            if n==0:
                return self.term(SuperPartition([[0],[]]))
            return self.term(SuperPartition([[n],[]])) - \
               self.sum(self.sgen_p_to_self(r)*self.term(SuperPartition([[n-r],[]]),1/(n+1)) \
                        for r in range(1,n+1)) - \
               self.sum(self.agen_p_to_self(r)*self.term(SuperPartition([[],[n-r]]),(r+1)/(n+1)) \
                        for r in range(n))

        def e_to_self_on_generator(self, n):
            r"""
            An Elementary generator in the Complete basis.

            INPUT:

            - ``n`` -- an integer

            The generators are ordered so that the non-positive integers
            are mapped to a fermionic generator and the positive
            integers are mapped to the symmetric generator.

            EXAMPLES::
            
                sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                sage: h.e_to_self_on_generator(2)
                h[; 1, 1] - h[; 2]
                sage: h.e_to_self_on_generator(-2)
                3*h[0; 1, 1] - 2*h[0; 2] - 2*h[1; 1] + h[2; ]
            """
            if n<=0:
                return self.agen_e_to_self(abs(n))
            else:
                return self.sgen_e_to_self(n)

        @cached_method
        def sgen_e_to_self(self, n):
            r"""
            A symmetric Elementary generator expanded in the Complete basis.

            INPUT:

            - ``n`` -- a positive integer

            EXAMPLES::

                sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                sage: h.sgen_e_to_self(4)
                h[; 1, 1, 1, 1] - 3*h[; 2, 1, 1] + h[; 2, 2] + 2*h[; 3, 1] - h[; 4]
            """
            if n==0:
                return self.term(SuperPartition([[],[]]))
            return self.sum(self.term(SuperPartition([[],la.to_list()]), \
                (-1)**(n-len(la))*factorial(len(la))/mul([factorial(a) for a in \
                    la.to_exp()])) for la in Partitions(n))

        @cached_method
        def agen_e_to_self(self, n):
            r"""
            A fermionic Elementary generator expanded in the Complete basis.

            INPUT:

            - ``n`` -- a non-negative integer

            This is equation (3.20) of [DLM06]_.

            EXAMPLES::

                sage: h = SymmetricFunctionsinSuperSpace(QQ).h()
                sage: h.agen_e_to_self(2)
                3*h[0; 1, 1] - 2*h[0; 2] - 2*h[1; 1] + h[2; ]
            """
            n = Integer(n)
            if n==0:
                return self.term(SuperPartition([[0],[]]))
            return self.sum((-1)**(n+r)*self.sgen_e_to_self(r)*\
                self.term(SuperPartition([[n-r],[]])) for r in range(n+1)) - \
                self.sum((-1)**(n+r)*self.agen_e_to_self(r)*\
                self.term(SuperPartition([[],[n-r]])) for r in range(n))

    h = Complete
    Homogeneous = Complete

    class Power(CombinatorialFreeModule, BindableClass):
        def __init__(self, SFSS):
            r"""
            TESTS::

                sage: p = SymmetricFunctionsinSuperSpace(QQ).Power()
                sage: TestSuite(p).run()
            """
            CombinatorialFreeModule.__init__(self, SFSS.base_ring(), \
                SuperPartitions(), prefix='p', bracket=False, \
                category=SFSS.MultiplicativeBases())

        def monomial_to_self_by_triangularity(self, m_expr):
            r"""
            Change a monomial expression to the power sum basis.

            The change of basis between the monomial and power sum basis
            elements have coefficients.

            INPUT:

            - ``m_expr`` -- an expression in the monomial basis

            EXAMPLES::

                sage: m = SymmetricFunctionsinSuperSpace(QQ).Monomial()
                sage: p = SymmetricFunctionsinSuperSpace(QQ).Power()
                sage: p.monomial_to_self_by_triangularity(m[[2],[1,1]])
                1/2*p[2; 1, 1] - 1/2*p[2; 2] - p[3; 1] + p[4; ]
            """
            out = self.zero()
            SFSS_m = m_expr.parent() # monomial basis
            while m_expr!=self.zero():
                (sp, c) = m_expr.trailing_item()
                ft = mul(factorial(p) for p in Partition(sp[1]).to_exp())
                out += self.term(sp, c/ft)
                m_expr -= c/ft*SFSS_m.power_to_self_on_basis(sp)
            return out

        @cached_method
        def h_to_self_on_generator(self, n):
            r"""
            A Complete generator expanded in the Power basis.

            INPUT:

            - ``n`` -- an integer

            The generators are ordered so that the non-positive integers
            are mapped to a fermionic generator and the positive
            integers are mapped to the symmetric generator.

            This is equation (3.61) and equation (3.62) of [DLM06]_.

            EXAMPLES::

                sage: p = SymmetricFunctionsinSuperSpace(QQ).p()
                sage: p.h_to_self_on_generator(-2)
                1/2*p[0; 1, 1] + 1/2*p[0; 2] + p[1; 1] + p[2; ]
                sage: p.h_to_self_on_generator(3)
                1/6*p[; 1, 1, 1] + 1/2*p[; 2, 1] + 1/3*p[; 3]
            """
            return self.sum(self.term(sp,1/sp.zee()) for sp \
                in SuperPartitions(abs(n),Integer(n<=0)))

        @cached_method
        def e_to_self_on_generator(self, n):
            r"""
            An Elementary generator expanded in the Power basis.

            INPUT:

            - ``n`` -- an integer

            The generators are ordered so that the non-positive integers
            are mapped to a fermionic generator and the positive
            integers are mapped to the symmetric generator.

            This is equation (3.61) and equation (3.62) of [DLM06]_.

            EXAMPLES::

                sage: p = SymmetricFunctionsinSuperSpace(QQ).p()
                sage: p.e_to_self_on_generator(-2)
                1/2*p[0; 1, 1] - 1/2*p[0; 2] - p[1; 1] + p[2; ]
                sage: p.e_to_self_on_generator(3)
                1/6*p[; 1, 1, 1] - 1/2*p[; 2, 1] + 1/3*p[; 3]
            """
            return self.sum(self.term(sp,sp.sign()/sp.zee()) for sp \
                in SuperPartitions(abs(n),Integer(n<=0)))

        def coproduct_on_generators(self, n):
            r"""
            The coproduct on the power sum generators.

            INPUT:

            - ``n`` -- an integer

            OUTPUT:

            - an element of the tensor square of the power basis

            EXAMPLES::

                sage: p = SymmetricFunctionsinSuperSpace(QQ).p()
                sage: p.coproduct_on_generators(-2)
                p[; ] # p[2; ] + p[2; ] # p[; ]
                sage: p.coproduct_on_generators(3)
                p[; ] # p[; 3] + p[; 3] # p[; ]
            """
            from sage.categories.all import tensor
            x = self.algebra_generators()[n]
            return tensor([self.one(),x])+tensor([x,self.one()])

        @lazy_attribute
        def coproduct(self):
            r"""
            Return the coproduct morphism in the basis ``self``.

            OUTPUT:

            - an element of the tensor square of the power basis

            EXAMPLES::

                sage: p = SymmetricFunctionsinSuperSpace(QQ).p()
                sage: p([[2],[1]]).coproduct()
                p[; ] # p[2; 1] + p[; 1] # p[2; ] + p[2; ] # p[; 1] + p[2; 1] # p[; ]
            """
            return self.algebra_morphism(self.coproduct_on_generators, \
                codomain = self.tensor_square())

        def antipode_on_basis(self, sp):
            r"""
            The antipode on a power basis element indexed by ``sp``.

            INPUT:

            - ``sp`` -- a super-partition

            OUPUT:

            - an element of the power sum basis

            EXAMPLES::

                sage: p = SymmetricFunctionsinSuperSpace(QQ).p()
                sage: p.antipode_on_basis(SuperPartition([[2,1],[2]]))
                -p[2, 1; 2]
                sage: p.antipode_on_basis(SuperPartition([[2],[1]]))
                p[2; 1]
            """
            return self.term(sp,(-1)**(sp.length()+binomial(sp.fermionic_degree(),2)))

        class Element(CombinatorialFreeModule.Element):

            def scalar(self, other):
                r"""
                The scalar product between ``self`` and ``other``.

                INPUT:

                - ``other`` -- an element of symmetric functions in super space

                The power sum elements are orthonormal

                EXAMAPLES::

                    sage: p = SymmetricFunctionsinSuperSpace(QQ).p()
                    sage: p[[],[3,1]].scalar(p[[1],[2,1]])
                    0
                    sage: p[[1],[2,1]].scalar(p[[1],[2,1]])
                    2
                """
                p = self.parent()
                py = p(other)
                return sum(sp.zee()*c*py.coefficient(sp) for (sp,c) in self)

            def omega(self):
                r"""
                The involution omega.

                The omega is computed by coercion to the power sum basis.

                EXAMPLES::

                    sage: p = SymmetricFunctionsinSuperSpace(QQ).p()
                    sage: p[[1,0],[2,2,1]].omega()
                    -p[1, 0; 2, 2, 1]
                    sage: all(p[[n],[]].omega()==p[[n],[]] for n in range(5))
                    True
                    sage: all(all(p[[],[n]].omega()==(-1)**n*p[[],[n]] for n in range(5)))
                    True
                """
                p = self.parent()
                return p.sum(sp.sign()*c*p(sp) for (sp, c) in self)

            def skew_by(self, other, side='right'):
                r"""
                Operation that is dual to multiplication in the scalar product

                INPUT:

                - ``other`` - a supersymmetric function

                - ``side`` - either 'right' or 'left' (default: 'right')

                OUTPUT:

                - a supersymmetric function

                EXAMPLES::

                    sage: p = SymmetricFunctionsinSuperSpace(QQ).p()
                    sage: p[-2,0,3,2].skew_by(p[-2])
                    -p[0; 3, 2]
                    sage: p[-2,0,3,2].skew_by(p[-2],'left')
                    p[0; 3, 2]
                    sage: p[-2,0,3,2].skew_by(p[0])
                    p[2; 3, 2]
                    sage: p[-2,0,3,2].skew_by(p[3])
                    3*p[2, 0; 2]
                """
                p = self.parent()
                if side=='right':
                    f = lambda sp,bd: p(sp)*other.homogeneous_bi_degree_component(bd)
                else:
                    f = lambda sp,bd: p(other).homogeneous_bi_degree_component(bd)*p(sp)
                return sum(self.scalar(f(sp,bd2))*p(sp)/sp.zee() \
                    for bd1 in self.all_bi_degrees() \
                    for bd2 in other.all_bi_degrees() \
                    for sp in SuperPartitions(bd1[0]-bd2[0], bd1[1]-bd2[1]))

    p = Power
